//
//  AppDelegate.swift
//  KakaoPartnerSDKSample
//
//  Created by lucas on 2020/04/24.
//  Copyright © 2020 apiteam. All rights reserved.
//

import UIKit
import KakaoSDKCommon
import KakaoSDKAuth
import KakaoPartnerSDKCommon

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {
    var window: UIWindow?
    var sampleVC: SDKSampleViewController!
    var partnerSampleVC: PartnerSDKSampleViewController!
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        // Override point for customization after application launch.
        
        // Override point for customization after application launch.
        UITabBarController.setPhase(phase: Properties.phase())
        
        window = UIWindow(frame: UIScreen.main.bounds)
        
        let tabBarController = UITabBarController()
        let systemFontAttributes = [NSAttributedString.Key.font: UIFont.systemFont(ofSize: 14.0)]
        UITabBarItem.appearance().setTitleTextAttributes(systemFontAttributes, for: .normal)
                
        self.sampleVC = SDKSampleViewController()
        self.sampleVC.navigationItem.rightBarButtonItems = [UIBarButtonItem(title: "Log", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.showLog)),
                                                            UIBarButtonItem(title: "Phase", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.changePhase))]
        self.sampleVC.setTitle(args:Properties.phase()?.rawValue)
        
        let firstNaviController = UINavigationController(rootViewController:self.sampleVC)
        firstNaviController.tabBarItem = UITabBarItem(title: "KakaoSDK", image:nil, tag: 0)
        
        self.partnerSampleVC = PartnerSDKSampleViewController()
        self.partnerSampleVC.navigationItem.rightBarButtonItems = [UIBarButtonItem(title: "Log", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.showLog)),
                                                       UIBarButtonItem(title: "Phase", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.changePhase))]
        self.partnerSampleVC.setTitle(args:Properties.phase()?.rawValue)
        
        let secondNaviController = UINavigationController(rootViewController:self.partnerSampleVC)
        secondNaviController.tabBarItem = UITabBarItem(title: "KakaoPartnerSDK", image:nil, tag: 1)
        
        
        tabBarController.viewControllers = [firstNaviController, secondNaviController]
        tabBarController.view.backgroundColor = UIColor.green

        window?.rootViewController = tabBarController
        window?.makeKeyAndVisible()
        
        NotificationCenter.default.addObserver(self, selector: #selector(hostChanged), name: Notification.Name("HostChanged"), object: nil)
        
        return true
    }
    
    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
        
        NotificationCenter.default.removeObserver(self, name: Notification.Name("HostChanged"), object: nil)
    }
    
    func application(_ app: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        if (AuthApi.isKakaoTalkLoginUrl(url)) {
            return AuthController.handleOpenUrl(url: url, options: options)
        }
        return false
    }
    
    @objc func hostChanged() {
        self.sampleVC.resetResultIndicator()
        self.sampleVC.setTitle(args:Properties.phase()?.rawValue)
        
        self.partnerSampleVC.resetResultIndicator()
        self.partnerSampleVC.setTitle(args:Properties.phase()?.rawValue)
    }
}

extension AppDelegate {
    static func templateId() -> (message:Int64, link:Int64) {
        switch Properties.phase() {
        case .Dev:
            return (message:151202, link:151203)
        case .Sandbox:
            return (message:682, link:683)
        case .Cbt:
            fallthrough
        case .Production:
            return (message:19383, link:19387)
        default:
            return (message:19383, link:19387)
        }
    }
}

extension UITabBarController {
    
    override open var canBecomeFirstResponder: Bool {
        return true
    }
    
    override open func motionBegan(_ motion: UIEvent.EventSubtype, with event: UIEvent?) {
        super.motionBegan(motion, with: event)
        
        if (motion == .motionShake) {
            print("shake was detected")
            self.showLog()
        }
    }
    
    @objc public func showLog() {
        let window = UIApplication.shared.keyWindow!
        
        let debugVC = SDKDebugViewController.shared
        let debugNaviController = SDKDebugNavigationController(rootViewController:debugVC)
        debugNaviController.modalPresentationStyle = .overCurrentContext
        debugNaviController.modalTransitionStyle = .crossDissolve
        

        if let modalViewController = window.rootViewController?.presentedViewController {
            if (modalViewController is SDKDebugNavigationController) {
                modalViewController.dismiss(animated: false, completion: nil)
            }
            else {
                modalViewController.present(debugNaviController, animated: false) {
                    debugVC.showLog()
                }
            }
        }
        else {
            window.rootViewController!.present(debugNaviController, animated: false) {
                debugVC.showLog()
            }
        }
    }
    
    @objc func changePhase(sender: Any) {
        var prevPhase : Phase = .Production
        if let storedPhase = Properties.phase() {
            prevPhase = storedPhase
        }
        
        let alert = UIAlertController(title: "Change Phase", message: "current:(\(prevPhase.rawValue))\nYou can change host", preferredStyle: .actionSheet)
        alert.addAction(UIAlertAction(title: "Dev", style:((prevPhase == .Dev) ? .destructive : .default) , handler:{ (UIAlertAction) in
            guard (prevPhase != .Dev) else { return }
            UITabBarController.setPhase(phase: .Dev, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Dev ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        
        alert.addAction(UIAlertAction(title: "Sandbox", style:((prevPhase == .Sandbox) ? .destructive : .default) , handler:{ (UIAlertAction) in
            
            guard (prevPhase != .Sandbox) else { return }
            UITabBarController.setPhase(phase: .Sandbox, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Sandbox ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        
        alert.addAction(UIAlertAction(title: "Cbt", style:((prevPhase == .Cbt) ? .destructive : .default) , handler:{ (UIAlertAction) in
            guard (prevPhase != .Cbt) else { return }
            UITabBarController.setPhase(phase: .Cbt, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Cbt ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        alert.addAction(UIAlertAction(title: "Production", style:((prevPhase == .Production) ? .destructive : .default) , handler:{ (UIAlertAction) in
            guard (prevPhase != .Production) else { return }
            UITabBarController.setPhase(phase: .Production, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Production ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        
        alert.addAction(UIAlertAction(title: "Close", style: .cancel, handler:{ (UIAlertAction) in
            print("User click Close button")
        }))
        
        self.present(alert, animated: true, completion: {
            print("completion block")
        })
    }
    
    static func setPhase(phase:Phase?, needRefresh:Bool = false) {
        if let phase = phase {
            switch phase {
            case .Dev:
                KakaoSDKCommon.initSDK(appKey: "25f3dfe2da25c29a406d8f55de1d7dbb", loggingEnable: true, hosts:Hosts(phase:phase))
            case .Sandbox:
                KakaoSDKCommon.initSDK(appKey: "2e3bcf1444acdc03f6267db1b05bb03e", loggingEnable: true, hosts:Hosts(phase:phase))
            case .Cbt:
                KakaoSDKCommon.initSDK(appKey: "9f9de684c354a72d2eb2a540a11441c2", loggingEnable: true, hosts:Hosts(phase:phase))
            case .Production:
                KakaoSDKCommon.initSDK(appKey: "9f9de684c354a72d2eb2a540a11441c2", loggingEnable: true, hosts:Hosts(phase:phase))
            }
            if (needRefresh) {
                Properties.setPhase(phase)
                Auth.shared.tokenManager.deleteToken()
                NotificationCenter.default.post(name: Notification.Name("HostChanged"), object: nil)
            }
        }
        else {
            KakaoSDKCommon.initSDK(appKey: "9f9de684c354a72d2eb2a540a11441c2", loggingEnable: true, hosts:Hosts(phase:.Production))
            Properties.setPhase(.Production)
        }
    }
}

